package com.myoa.engineering.crawl.shopping.crawlhandler.parser;

import com.myoa.engineering.crawl.shopping.domain.entity.v1.PpomppuArticle;
import com.myoa.engineering.crawl.shopping.support.dto.constant.PpomppuBoardName;
import com.myoa.engineering.crawl.shopping.util.DateTimeUtils;
import com.myoa.engineering.crawl.shopping.util.NumberUtils;
import com.myoa.engineering.crawl.shopping.util.TestDataUtils;
import io.micrometer.core.instrument.util.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.junit.jupiter.api.Test;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

class PpomppuArticleParserV2Test {

    @Test
    void test1() {
        String data = TestDataUtils.fileToString("testdata/zboard/file1.html");
        Document document = Jsoup.parse(data);
        Elements trList = document.getElementById("revolution_main_table").getElementsByTag("tr");

        List<PpomppuArticle> articles = trList.stream()
                                              .filter(this::isRealArticle)
                                              .map(this::parse)
                                              .toList();
        System.out.println(articles.size());
        System.out.println(articles.get(0));
        System.out.println(articles.get(1));
    }

    private boolean isRealArticle(Element tr) {
        Elements tdList = tr.getElementsByTag("td");
        if (tdList.size() != 6) {
            return false;
        }

        if (!hasOnlyNumeric(tdList.get(0))) {
            return false;
        }

        return true;
    }


    Pattern pattern_numeric = Pattern.compile("\\d+");

    private boolean hasOnlyNumeric(Element td) {
        return pattern_numeric.matcher(td.text()).matches();
    }

    public PpomppuArticle parse(Element tr) {
        Elements tdList = tr.getElementsByTag("td");
        Long articleId = Long.parseLong(tdList.get(0).text());

        String title = tdList.get(2).text();
        String articleUrl = parseArticleUrl(tdList.get(2).getElementsByTag("a").attr("href"));
        PpomppuBoardName boardName = parseBoardName(title);
        Integer recommended = parseRecommended(tdList.get(4));
        Integer hit = NumberUtils.parseInt(tdList.get(5).text(), 0);


        ZonedDateTime registeredAt = DateTimeUtils.parse(tdList.get(3).text());

        return PpomppuArticle.builder()
                             .articleId(articleId)
                             .title(title)
                             .boardName(boardName)
                             .articleUrl(articleUrl)
                             .recommended(recommended)
                             .hit(hit)
                             .registeredAt(registeredAt)
                             .build();
    }

    public Long parseArticleId(Element td) {
        return Long.parseLong(td.text().trim());
    }

    public static Integer parseRecommended(Element td) {
        final String voteString = td.text();
        if (StringUtils.isEmpty(voteString)) {
            return null;
        }

        final int voteUp = Integer.parseInt(td.text().split(" - ")[0]);
        final int voteDown = Integer.parseInt(td.text().split(" - ")[1]);
        int recommended = voteUp - voteDown;
        return recommended;
    }

    public static String parseArticleUrl(String data) {
        return PpomppuBoardName.ofViewPageUrl(data);
    }


    Pattern patternBoardName = Pattern.compile("\\[(.+?)\\]");

    public PpomppuBoardName parseBoardName(String fullTitle) {
        Matcher matcher = patternBoardName.matcher(fullTitle);
        String lastMatched = null;
        while (matcher.find()) {
            lastMatched = matcher.group(1);
        }
        return PpomppuBoardName.ofBoardName(lastMatched, true);
    }

    @Test
    void test2() {
        PpomppuBoardName boardName = parseBoardName("[자사몰]푸마 메쉬 폼스트라이프 [에디션] 5종 [세트] (18,220원/무료)6 [의류/잡화]");
        System.out.println(boardName);
    }
}